"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const path_1 = __importDefault(require("path"));
const validation_1 = __importDefault(require("../errors/validation"));
const command_1 = __importDefault(require("../models/command"));
class Cordova extends command_1.default {
    constructor() {
        super(...arguments);
        this.indexAdd = `<!DOCTYPE html>
<!--
    Licensed to the Apache Software Foundation (ASF) under one
    or more contributor license agreements.  See the NOTICE file
    distributed with this work for additional information
    regarding copyright ownership.  The ASF licenses this file
    to you under the Apache License, Version 2.0 (the
    "License"); you may not use this file except in compliance
    with the License.  You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

    Unless required by applicable law or agreed to in writing,
    software distributed under the License is distributed on an
    "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
     KIND, either express or implied.  See the License for the
    specific language governing permissions and limitations
    under the License.
-->
<html>
    <head>
        <!--
        Customize this policy to fit your own app's needs. For more guidance, see:
            https://github.com/apache/cordova-plugin-whitelist/blob/master/README.md#content-security-policy
        Some notes:
            * gap: is required only on iOS (when using UIWebView) and is needed for JS->native communication
            * https://ssl.gstatic.com is required only on Android and is needed for TalkBack to function properly
            * Disables use of inline scripts in order to mitigate risk of XSS vulnerabilities. To change this:
                * Enable inline JS: add 'unsafe-inline' to default-src
        -->
        <meta http-equiv="Content-Security-Policy" content="default-src * data: gap: cdvfile: 'unsafe-eval'; style-src * 'unsafe-inline'; media-src *">
        <meta name="format-detection" content="telephone=no">
        <meta name="msapplication-tap-highlight" content="no">`;
        this.defaultComponent = {
            component: path_1.default.resolve(__dirname, '..', 'cordova'),
            command: 'create',
            prefix: 'plat-cordova-',
        };
    }
    help(command) {
        return super.help(command).then(() => {
            this.ui.help(`Here is the \`cordova\` help:`);
            this.ui.help(`..........................................................`);
            return this.exec(this.args);
        });
    }
    generalHelp(command) {
        let baseCommand = this.buildFullCommand().join(' ');
        this.ui.help(`
  This command will run \`cordova\` commands in the context of your cordova project directory.

  General Usage:

    ${baseCommand} <command> [...options]`);
    }
    run() {
        return this.exec(this.args);
    }
    exec(args) {
        if (args[0] === 'cordova') {
            args = args.slice(1);
        }
        let arg = args[0], promise;
        if (arg === 'build' || arg === 'compile') {
            promise = this.modifyIndex();
        }
        else {
            promise = Promise.resolve(null);
        }
        return promise.then(() => {
            return this.process.exec('cordova', args, {
                cwd: path_1.default.resolve(this.project.root, 'cordova'),
            });
        });
    }
    modifyIndex() {
        let cordova = '<script type="text/javascript" src="cordova.js"></script>', haveCordova = false, scriptStart = -1, file = path_1.default.resolve(this.project.root, 'cordova/www/index.html');
        return this.file.read(file).then((data) => {
            let eol = this.file.eol(data), lines = data.split(eol);
            this.removeBase(lines);
            this.addIndexHead(lines, data);
            haveCordova = lines.some((line, index) => {
                if (line.indexOf('src="cordova.js"') > -1) {
                    return true;
                }
                if (scriptStart === -1 && line.indexOf('<script') > -1) {
                    scriptStart = index;
                }
            });
            if (!haveCordova) {
                let spaces = (/^(\s*)/.exec(lines[scriptStart]) || ['', ''])[1];
                lines.splice(scriptStart, 0, spaces + cordova);
            }
            return this.file.write(file, lines.join(eol));
        });
    }
    removeBase(lines) {
        let index = this.findBase(lines);
        if (index === -1) {
            return;
        }
        lines.splice(index, 1);
    }
    addIndexHead(lines, data) {
        if (data.indexOf('http-equiv="Content-Security-Policy"') > -1) {
            return;
        }
        this.ui.info('Adding cordova tags to index.html');
        let index = this.findHead(lines);
        if (index === -1) {
            throw new validation_1.default('You need a `<head>` tag.');
        }
        lines.splice(0, index + 1, this.indexAdd);
    }
    findHead(lines) {
        let index = -1;
        lines.some((line, i) => {
            if (line.indexOf('<head>') > -1) {
                index = i;
            }
            return index > -1;
        });
        return index;
    }
    findBase(lines) {
        let base = '<base href="/">', index = -1;
        lines.some((line, i) => {
            if (line.indexOf(base) > -1) {
                index = i;
            }
            return index > -1;
        });
        return index;
    }
}
Cordova.commandName = 'cordova';
exports.default = Cordova;
