"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const command_1 = __importDefault(require("../models/command"));
const notfound_1 = __importDefault(require("../errors/notfound"));
class Invalid extends command_1.default {
    help(command) {
        if (this.commands.length > 0 && this.options.help) {
            return this.validate();
        }
        this.ui.help(`Platypi CLI \`v${this.version}\` Help`);
        let commands = this.getCommands(), baseCommand = this.buildFullCommand().join(' ');
        this.ui.help(`
  General Usage:

    plat <command> [...options]

  Commands:
`);
        commands.forEach((c) => {
            this.ui.help(`    ${baseCommand} ${c} -h`);
        });
        this.ui.help('');
        this.optionsHelp(command);
    }
    defineOptions() {
        this.option('version', {
            aliases: ['v'],
            description: 'Print the version of this CLI'
        });
    }
    validate() {
        let command = this.commands[0];
        if (this.utils.isString(command)) {
            throw new notfound_1.default(`Cannot execute \`${this.buildFullCommand().join(' ')}\`, \`${command}\` is not a valid command.`);
        }
    }
    run() {
        if (!!this.options.version) {
            this.ui.help(`Platypi CLI \`v${this.version}\``);
        }
        else {
            this.help('');
        }
    }
    aliasesHelp(command) { }
    get version() {
        return this.project.cliPackage().version;
    }
    getCommands() {
        return this.utils.keys(require('require-all')({
            dirname: __dirname,
            filter: /^((?!invalid).*)\.js$/
        }));
    }
}
exports.default = Invalid;
Invalid.commandName = 'invalid';
