"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const path = __importStar(require("path"));
const base_1 = __importDefault(require("../models/base"));
const notfound_1 = __importDefault(require("../errors/notfound"));
const fileutils_1 = __importDefault(require("../models/fileutils"));
const utils_1 = require("../utils/utils");
let win32 = process.platform === 'win32';
class Environment extends base_1.default {
    constructor(options) {
        super(options);
        this.fileUtils = this.instantiate(fileutils_1.default, options);
    }
    command(defaults, command = '', parent) {
        defaults = this.utils.clone(defaults);
        let component = this.parseComponent(defaults, command), prefix = defaults.prefix;
        return this._commands(component.component, prefix).then((response) => {
            if (!this.utils.isEmpty(response.commands)) {
                return response;
            }
            component.command = component.component;
            component.component = defaults.component;
            return this._commands(component.component, prefix);
        }).then((response) => {
            let commands = response.commands, Com = commands[component.command];
            if (!this.utils.isObject(Com)) {
                Com = utils_1.findCommand(this.utils.values(commands), component.command);
            }
            try {
                return this.instantiate(Com, {
                    env: this,
                    parent: parent,
                    directory: response.directory
                });
            }
            catch (e) {
                if (e.message.indexOf('Cannot find module') > -1) {
                    throw new notfound_1.default('Unrecognized component: `' + command + '`');
                }
                throw e;
            }
        });
    }
    listCommands(defaults, command = '') {
        defaults = this.utils.clone(defaults);
        let component = this.parseComponent(defaults, command), prefix = defaults.prefix;
        return this._commands(component.component, prefix).then((response) => {
            if (!this.utils.isEmpty(response.commands)) {
                return response;
            }
            component.command = component.component;
            component.component = defaults.component;
            return this._commands(component.component, prefix);
        }).then((response) => {
            return this.utils.keys(response.commands);
        });
    }
    parseComponent(defaults, command) {
        if (this.utils.isEmpty(command)) {
            command = defaults.command;
        }
        let components = command.split(/:(?!\\|\/)/), component;
        if (components.length > 0) {
            component = components.shift();
        }
        else {
            component = defaults.component;
        }
        command = components.shift();
        if (this.utils.isEmpty(command)) {
            command = defaults.command;
        }
        return { component, command, prefix: defaults.prefix };
    }
    getNpmPaths() {
        let paths = [];
        process.cwd().split(path.sep).forEach((part, index, parts) => {
            let lookup = path.join.apply(path, parts.slice(0, index + 1).concat(['node_modules']));
            if (!win32) {
                lookup = '/' + lookup;
            }
            paths.push(lookup);
        });
        if (process.env.NODE_PATH) {
            paths = this.utils.compact(process.env.NODE_PATH.split(path.delimiter)).concat(paths);
        }
        else {
            if (win32) {
                paths.push(path.join(process.env.APPDATA, 'npm/node_modules'));
            }
            else {
                paths.push('/usr/lib/node_modules');
            }
        }
        return paths.reverse();
    }
    _commands(component, prefix) {
        return this._find(this.getNpmPaths(), component, prefix).then((info) => {
            component = info.component;
            let values = info.values, commands = this.fileUtils.requireAll(component, values);
            this.utils.forEach(commands, (command, name) => {
                command.commandName = name;
            });
            return {
                commands: commands,
                directory: component
            };
        }, () => {
            return {
                commands: {},
                directory: ''
            };
        });
    }
    _find(paths, component, prefix) {
        let absolute = component, isRelative = !utils_1.isAbsolute(absolute);
        if (isRelative) {
            absolute = paths.shift() + path.sep + prefix + component;
        }
        return this.fileUtils.dir(absolute, [
            'templates',
            'node_modules',
            /^_.*$/,
            /^\..*$/
        ]).then((values) => {
            return { component: absolute, values };
        }, (err) => {
            if (this.utils.isEmpty(paths) || !isRelative) {
                throw err;
            }
            return this._find(paths, component, prefix);
        });
    }
}
exports.default = Environment;
