"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const path = __importStar(require("path"));
const generator_1 = __importDefault(require("../../models/generator"));
const index_1 = __importDefault(require("../repository/index"));
const index_2 = __importDefault(require("../service/index"));
const index_3 = __importDefault(require("../viewcontrol/index"));
const index_4 = __importDefault(require("../cordova/index"));
let validate = require('validate-npm-package-name');
class AppGenerator extends generator_1.default {
    defineOptions() {
        this.option('name', {
            aliases: ['n'],
            description: `The name of the app`,
        });
        this.option('dir', {
            aliases: ['d'],
            description: `Specify the relative path to a directory in which to create the app`,
            defaults: '',
        });
        this.option('cordova', {
            description: `Don't create a cordova app`,
            defaults: true,
        });
        this.option('gitignore', {
            description: `Don't create a .gitignore`,
            defaults: true,
        });
    }
    askQuestions() {
        let options = this.options;
        return this.promptName(options.name)
            .then((name) => {
            options.name = name;
            if (this.utils.isEmpty(options.dir)) {
                name = this.utils.camelCase(name).toLowerCase();
            }
            return this.promptDir(options.dir, name);
        })
            .then((dir) => {
            options.dir = dir;
            return this.promptCordova(options.cordova);
        })
            .then((cordova) => {
            options.cordova = cordova;
            return this.promptGitIgnore(options.gitignore);
        })
            .then((gitignore) => {
            options.gitignore = gitignore;
        });
    }
    promptCordova(cordova) {
        if (!cordova) {
            return Promise.resolve(cordova);
        }
        return this.ui
            .prompt([
            {
                name: 'cordova',
                default: true,
                type: 'confirm',
                message: `Should we create a Cordova project?`,
            },
        ])
            .then((answer) => {
            return answer.cordova;
        });
    }
    promptGitIgnore(gitignore) {
        if (!gitignore) {
            return Promise.resolve(gitignore);
        }
        return this.ui
            .prompt([
            {
                name: 'gitignore',
                default: true,
                type: 'confirm',
                message: `Should we create a .gitignore?`,
            },
        ])
            .then((answer) => {
            return answer.gitignore;
        });
    }
    promptName(name = '') {
        name = name.trim();
        if (!this.utils.isEmpty(name)) {
            this.title = name;
            name = this.utils.kebabCase(name).toLowerCase();
            let valid = validate(name);
            if (this.utils.isArray(valid.errors) ||
                this.utils.isArray(valid.warnings)) {
                this.ui.warn('');
                this.utils.forEach(valid.errors, (err) => {
                    this.ui.warn(err);
                });
                this.utils.forEach(valid.warnings, (err) => {
                    this.ui.warn(err);
                });
                this.ui.warn('');
            }
            else {
                return Promise.resolve(name);
            }
        }
        return this.ui
            .prompt([
            {
                name: 'name',
                type: 'input',
                message: `What is the name of your app?`,
            },
        ])
            .then((answer) => {
            return this.promptName(answer.name);
        });
    }
    promptDir(dir = '', defaults) {
        dir = dir.trim();
        if (!this.utils.isEmpty(dir)) {
            return Promise.resolve(dir);
        }
        return this.ui
            .prompt([
            {
                name: 'dir',
                default: defaults,
                type: 'input',
                message: `Where should this app be created?`,
            },
        ])
            .then((answer) => {
            return answer.dir;
        });
    }
    run() {
        this.destRoot(this.options.dir + '/app');
        this.project.root = this.options.dir;
        this.ui.debug('Generating the `default` app');
        let destRoot = path.resolve(this.destRoot(), '..'), genOptions = {
            env: this.env,
            directory: this.directory,
            destRoot: destRoot,
        }, name = this.options.name, vcGenerator = this.instantiate(index_3.default, genOptions), repoGenerator = this.instantiate(index_1.default, genOptions), svcGenerator = this.instantiate(index_2.default, genOptions), cordovaGenerator = this.instantiate(index_4.default, genOptions), vcName = 'home', title = this.utils.startCase(this.title), options = {
            appName: name,
            appTitle: title,
            vcName: vcName,
        };
        cordovaGenerator.options = {
            id: name,
            name: title,
        };
        let promises = [
            this.render('package.json', '../package.json', options),
            this.render('README.md', '../README.md', options),
            this.render('tsconfig.json', '../tsconfig.json', options),
            this.render('index.html', 'index.html', options),
            this.file.copy(this.srcRoot() + '/favicon.ico', this.destRoot() + '/favicon.ico'),
            this.render('main.ts', 'src/main.ts', options),
            this.render('server.ts', '../server/server.ts', options),
            this.render('app/app.ts', 'src/app/app.ts', options),
            this.render('styles/main.less', 'styles/main.less', options),
            this.render('model/models.d.ts', 'src/models/models.d.ts', options),
            this.mkdirDest('src/attributecontrols', 'src/injectables', 'src/templatecontrols', 'lib', 'fonts', 'images', '../test/unit'),
        ];
        if (this.options.gitignore) {
            promises.push(this.render('gitignore', '../.gitignore', options));
        }
        repoGenerator.options = svcGenerator.options = {
            name: 'base',
            extends: false,
        };
        return Promise.all(promises)
            .then(() => {
            vcGenerator.options = {
                name: vcName,
                less: true,
                html: true,
            };
            return vcGenerator.run();
        })
            .then(() => {
            vcGenerator.options = {
                name: 'base',
                extends: false,
                register: false,
                less: false,
                html: false,
            };
            return vcGenerator.run();
        })
            .then(() => {
            return repoGenerator.run();
        })
            .then(() => {
            return svcGenerator.run();
        })
            .then(() => {
            if (this.options.cordova) {
                return cordovaGenerator.run();
            }
        });
    }
}
exports.default = AppGenerator;
