"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const path = __importStar(require("path"));
const Handlebars = __importStar(require("handlebars"));
const swag_1 = require("swag");
const command_1 = __importDefault(require("./command"));
const utils_1 = require("../utils/utils");
swag_1.registerHelpers(Handlebars);
class Generator extends command_1.default {
    constructor(options) {
        super(options);
        this._srcRoot = '';
        this._destRoot = '';
        this._finishRender = Promise.resolve();
        this.env = options.env;
        this.directory = options.directory;
        this.srcRoot(path.resolve(options.directory, 'templates'));
        let project = this.project, root;
        if (this.utils.isObject(project)) {
            root = project.root;
        }
        this.destRoot(root || options.destRoot || process.cwd());
    }
    render(source, destination, context) {
        let src = this.getPath(this._srcRoot, source), dest = this.getPath(this._destRoot, destination);
        let options = this.utils.extend({
            context: context,
            encoding: 'utf8',
        }, context);
        return (this._finishRender = this._finishRender
            .then(() => {
            return this.file.read(src, options);
        })
            .then((data) => {
            data = Handlebars.compile(data, {
                noEscape: true,
            })(options.context);
            return this.file.read(dest, options).then(() => {
                this.ui.warn(`The file \`${path.normalize(dest.replace(this.project.root, ''))}\` already exists.`);
                return this.ui
                    .prompt([
                    {
                        type: 'confirm',
                        default: true,
                        name: 'force',
                        choices: ['Y', 'n'],
                        message: 'Are you sure you want to overwrite it?',
                    },
                ])
                    .then((answer) => {
                    if (!answer.force) {
                        return;
                    }
                    return this.file.write(dest, data, options);
                });
            }, (err) => {
                return this.file.write(dest, data, options);
            });
        }));
    }
    copy(src, dest) {
        src = this.getPath(this._srcRoot, src);
        dest = this.getPath(this._destRoot, dest);
        return this.file.copy(src, dest);
    }
    mkdirDest(...dirs) {
        return this.file.mkdir.apply(this, dirs.map((dir) => {
            return this.getPath(this._destRoot, dir);
        }));
    }
    srcRoot(source) {
        return (this._srcRoot = this.getPath(this._srcRoot, source));
    }
    destRoot(dest) {
        return (this._destRoot = this.getPath(this._destRoot, dest));
    }
    mapLines(handler, data) {
        let eol = this.file.eol(data), lines = data.split(eol);
        return this.utils.map(lines, handler).join(eol);
    }
    getPath(source, append) {
        if (this.utils.isEmpty(source) || utils_1.isAbsolute(append)) {
            source = append;
        }
        else {
            source = path.resolve(source, append || '.');
        }
        return source;
    }
}
exports.default = Generator;
