"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const path = __importStar(require("path"));
const base_1 = __importDefault(require("./base"));
const lodash_1 = require("lodash");
const fileutils_1 = __importDefault(require("./fileutils"));
let findup = require('findup'), stringify = require('json-stable-stringify');
class Project extends base_1.default {
    constructor(options) {
        super(options);
        this.finishReading = Promise.resolve(null);
        this.finishWriting = Promise.resolve(null);
        let cliPackage = (this.cliPkg = require('../../package.json'));
        this.bin = this.utils.keys(cliPackage.bin)[0];
        this.root = options.root;
        this.pkg = this.utils.cloneDeep(options.pkg);
        this.file = this.instantiate(fileutils_1.default, options);
    }
    get name() {
        return this.pkg.name;
    }
    get pkgLocation() {
        return path.resolve(this.root, 'package.json');
    }
    static project(ui, root) {
        return Project.closestPackage(ui, root).then((info) => {
            ui.debug(`Closest project found at ${info.directory}`);
            return new Project({
                root: info.directory,
                pkg: info.pkg,
                ui: ui,
            });
        });
    }
    static closestPackage(ui, root) {
        return Promise.all([
            Project.closestConfig(ui, root, 'package.json'),
            Project.closestConfig(ui, root, 'platypi.json'),
        ]).then((values) => {
            let pkg = values[0], platypi = values[1], pkgError = !lodash_1.isString(pkg.directory), platypiError = !lodash_1.isString(platypi.directory);
            if (pkgError && platypiError) {
                return {
                    directory: undefined,
                    pkg: {},
                };
            }
            else if (pkgError) {
                pkg = {
                    directory: platypi.directory,
                    pkg: {},
                };
            }
            else if (platypiError) {
                platypi = {
                    directory: pkg.directory,
                    pkg: {},
                };
            }
            return {
                directory: pkg.directory,
                pkg: lodash_1.merge(pkg.pkg, { platypi: platypi.pkg }),
            };
        });
    }
    static closestConfig(ui, root, configName) {
        return new Promise((resolve, reject) => {
            ui.debug(`Searching for ${configName} at and above ${root}`);
            findup(root, configName, (err, directory) => {
                if (lodash_1.isObject(err)) {
                    resolve(Project.handleError(ui, root, err));
                    return;
                }
                resolve(directory);
            });
        }).then((directory) => {
            if (!lodash_1.isString(directory)) {
                return directory;
            }
            let config = path.join(directory, configName);
            ui.debug(`Reading ${config}`);
            return {
                directory: directory,
                pkg: lodash_1.cloneDeep(require(config)),
            };
        });
    }
    static handleError(ui, root, err) {
        if (lodash_1.isObject(err) && /not found/i.test(err.message)) {
            ui.debug(`No project found at or up from: \`${root}\``);
            return {
                directory: undefined,
                pkg: undefined,
            };
        }
        else {
            return err;
        }
    }
    getConfig(property) {
        let config = this.pkg.platypi || {};
        return config[property];
    }
    cliPackage() {
        return this.utils.cloneDeep(this.cliPkg);
    }
    package() {
        return this.utils.cloneDeep(this.pkg);
    }
    addDependencies(deps, dev) {
        return this.extendPackageProperty((dev ? 'devD' : 'd') + 'ependencies', deps);
    }
    addScripts(scripts) {
        return this.extendPackageProperty('scripts', scripts);
    }
    extendPackageProperty(property, value) {
        return (this.finishWriting = this.finishWriting
            .then(() => {
            return this.readLocalPackage();
        })
            .then((pkg) => {
            this.utils.extend(pkg[property], value);
            pkg[property] = JSON.parse(stringify(pkg[property]));
            return this.writeLocalPackage(pkg);
        }));
    }
    readLocalPackage() {
        return this.file.read(this.pkgLocation).then((pkg) => {
            return JSON.parse(pkg);
        });
    }
    writeLocalPackage(pkg) {
        return this.file.write(this.pkgLocation, JSON.stringify(pkg, null, 2));
    }
}
exports.default = Project;
