"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const chai_1 = require("chai");
const cli_1 = __importDefault(require("../../../src/cli/cli"));
const create_mock_1 = __importDefault(require("../mock/create.mock"));
const ui_mock_1 = __importDefault(require("../mock/ui.mock"));
chai_1.use(require('chai-as-promised'));
describe('cli', () => {
    let cli;
    beforeEach(() => {
        cli = new cli_1.default({
            ui: new ui_mock_1.default()
        });
    });
    it('should run a command when it is matched', (done) => {
        let ranCreate = false, Create = create_mock_1.default(() => {
            ranCreate = true;
        });
        chai_1.expect(cli.run({
            commands: {
                create: Create
            },
            args: [Create.commandName]
        })).to.eventually.be.an('undefined').notify(() => {
            chai_1.expect(ranCreate).to.be.ok;
            done();
        });
    });
    it('should run a command when its alias is matched', (done) => {
        let ranCreate = false, Create = create_mock_1.default(() => {
            ranCreate = true;
        });
        chai_1.expect(cli.run({
            commands: {
                create: Create
            },
            args: [Create.aliases[0]]
        })).to.eventually.be.an('undefined').notify(() => {
            chai_1.expect(ranCreate).to.be.ok;
            done();
        });
    });
    it('should return 0 when command not found', (done) => {
        chai_1.expect(cli.run({
            commands: {},
            args: ['noop']
        })).to.eventually.equal(0).notify(done);
    });
    it('should return 0 when erroring', () => {
        chai_1.expect(cli.error(new Error())).to.equal(0);
    });
});
