"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const chai_1 = require("chai");
const sinon_1 = require("sinon");
const command_1 = __importDefault(require("../../../src/models/command"));
const ui_mock_1 = __importDefault(require("../mock/ui.mock"));
const notimplemented_1 = __importDefault(require("../../../src/errors/notimplemented"));
const validation_1 = __importDefault(require("../../../src/errors/validation"));
chai_1.use(require('chai-as-promised'));
chai_1.use(require('sinon-chai'));
describe('Command', () => {
    let command;
    beforeEach(() => {
        command = new command_1.default({
            ui: new ui_mock_1.default(),
        });
    });
    it('should throw a NotImplementedError when trying to run', (done) => {
        chai_1.expect(command.validateAndRun(['']))
            .to.eventually.rejectedWith(notimplemented_1.default)
            .notify(done);
    });
    it('should throw a ValidationError when command is invalid', (done) => {
        sinon_1.stub(command, 'validate').callsFake(() => {
            return false;
        });
        chai_1.expect(command.validateAndRun([''])).to.eventually.rejectedWith(validation_1.default);
        chai_1.expect(command.validateAndRun(['create']))
            .to.eventually.rejectedWith(validation_1.default)
            .notify(done);
    });
});
